@@ -275,10 +275,17 @@ namespace Beagle.Daemon {
 			
 			ICollection added_uris = null;
 
+			// Index listeners never return any initial matches.
+			if (change_data == null && query.IsIndexListener)
+				return;
+
 			if (change_data != null) {
 				
-				if (change_data.RemovedUris != null) 
+				if (change_data.RemovedUris != null) {
+					foreach (Uri uri in change_data.RemovedUris)
+						Log.Debug ("Signalling removal of {0}", uri);
 					query_result.Subtract (change_data.RemovedUris);
+				}
 
 				// If nothing was added, we can safely return now: this change
 				// cannot have any further effect on an outstanding live query.
@@ -287,6 +294,24 @@ namespace Beagle.Daemon {
 					return;
 
 				added_uris = change_data.AddedUris;
+
+				// If this is an index listener, we don't need to do a query:
+				// we just build up synthethic hits and add them unconditionally.
+				if (query.IsIndexListener) {
+					ArrayList synthetic_hits = new ArrayList ();
+					foreach (Uri uri in added_uris) {
+						if (our_uri_filter != null && ! our_uri_filter (uri))
+							continue;
+						Hit hit = new Hit ();
+						hit.Uri = uri;
+						if (our_hit_filter != null && ! our_hit_filter (hit))
+							continue;
+						synthetic_hits.Add (hit);
+					}
+					if (synthetic_hits.Count > 0)
+						query_result.Add (synthetic_hits);
+					return;
+				}
 			}
 			
 			Driver.DoQuery (query, 
@@ -548,6 +573,8 @@ namespace Beagle.Daemon {
 
 					if (queryable.PreAddIndexableHook (generated))
 						queryable.AddIndexable (generated);
+					else
+						generated.Cleanup ();
 					
 					// We keep adding indexables until a flush goes through.
 				} while (! queryable.ConditionalFlush ());
@@ -755,6 +782,8 @@ namespace Beagle.Daemon {
 
 		protected void AddIndexable (Indexable indexable)
 		{
+			indexable.Source = QueryDriver.GetQueryable (this).Name;
+
 			lock (request_lock)
 				pending_request.Add (indexable);
 
@@ -918,8 +947,10 @@ namespace Beagle.Daemon {
 						if (please_add_a_new_task) {
 							//Logger.Log.Debug ("Adding child {0}", child.Uri);
 							Scheduler.Task task = NewAddTask (child);
+							task.SubPriority = 1;
 							ThisScheduler.Add (task);
-						}
+						} else
+							child.Cleanup ();
 					}
 				}
 			}
